var colors = require('colors')
var path = require('path')
var express = require('express')
var bodyParser = require('body-parser')
var textile = require('textile-js')
var config = require('config')
var app = express()
const PORT = config.get('Application.port')
const ID_CHAMP_ELIGIBLE = config.get('Redmine.champEligible')
const CHAMPS_RETENUS = config.get('Redmine.champRetenus')
const PROJETS_CENSURES = config.get('Redmine.projetsCendures')
var MAX_PROJETS
if (config.has('Redmine.maxProjets')) { MAX_PROJETS = config.get('Redmine.maxProjets') } else { MAX_PROJETS = 100 }
if (config.has('IHM.images')) { app.locals.imagesHeader = config.get('IHM.images') }
if (config.has('Redmine.pagesWiki')) { app.locals.wikiPages = config.get('Redmine.pagesWiki') }
app.locals.redminePath = config.get('Redmine.url')
app.locals.appTitle = config.get('IHM.titre')
app.set('view engine', 'pug')
app.use(bodyParser.urlencoded({ extended: true }))
app.use('/static', express.static(path.join(__dirname, 'static')))
app.use(express.static(path.join(__dirname, 'static')))

// Configuration de la connexion à Redmine
var Redmine = require('node-redmine')
var redmine = new Redmine(app.locals.redminePath, {apiKey: require('./redmineAPIKey.key').key})

// Pour remplacer les accents
String.prototype.sansAccent = function () {
  var accent = [
    /[\300-\306]/g, /[\340-\346]/g, // A, a
    /[\310-\313]/g, /[\350-\353]/g, // E, e
    /[\314-\317]/g, /[\354-\357]/g, // I, i
    /[\322-\330]/g, /[\362-\370]/g, // O, o
    /[\331-\334]/g, /[\371-\374]/g, // U, u
    /[\321]/g, /[\361]/g, // N, n
    /[\307]/g, /[\347]/g // C, c
  ]
  var noaccent = ['A', 'a', 'E', 'e', 'I', 'i', 'O', 'o', 'U', 'u', 'N', 'n', 'C', 'c']
  var str = this
  for (var i = 0; i < accent.length; i++) {
    str = str.replace(accent[i], noaccent[i])
  }
  return str
}

// variable globale contenant les projets
var listeProjets = {}

// fonction vérifiant l'éligibilité des projets
function eligibiliteProjet (projet) {
  var ineligible = 0

  // Si certains projets sont secrets, peu importe leur éligibilité au récapitulatif CNIL
  if (PROJETS_CENSURES.indexOf(projet.id) >= 0) { console.log('[PROJET] ' + 'sauté'.red + ': ' + projet.name); ineligible = 1 }

  if (ineligible === 0) {
    // Si le projet n'est pas éligible à une consultation CNIL, on passe au suivant
    for (var j = 0; j < projet.custom_fields.length; j++) {
      if (projet.custom_fields[j].id == ID_CHAMP_ELIGIBLE) {
        if (projet.custom_fields[j].value != 1) {
          ineligible = 1
          console.log('[PROJET] ' + 'non éligible'.yellow + ': ' + projet.name)
        }
      }
    }
  }
  if (ineligible === 0) { console.log('[PROJET] ' + 'éligible'.green + ': ' + projet.name) }
  return ineligible
}

// fonction permettant de renvoyer la page des projets introuvables le cas échéant
function presenceProjets (req, res, listeProjets) {
  if (Object.keys(listeProjets).length === 0) {
    console.log('[PROJET] Aucun projet éligible!'.red)
    res.render('erreur', {
      erreur: 'projet_introuvable',
      title: 'Aucun projet trouvé',
      message: 'Aucun projet trouvé!'
    })
    return 0
  }
}

// fonction permettant de renvoyer la liste des projets avec les champs retenus
function trouveProjets (req, res, next) {
  listeProjets = {}
  redmine.projects({limit: MAX_PROJETS}, function (err, data) {
    if (err) {
      console.log('[ERREUR REDMINE] '.red + err.red)
      if (err.indexOf('EAI_AGAIN') !== -1) {
        res.render('erreur', {
          erreur: 'timeout',
          title: 'Erreur connexion Redmine',
          message: 'Erreur de connexion au Redmine!'
        })
      } else {
        res.render('erreur', {
          erreur: 'connexion_redmine',
          title: 'Erreur connexion Redmine',
          message: 'Erreur de connexion au Redmine!'
        })
      }
      return 0
    }
    for (var i in data.projects) {
      var projet = data.projects[i]
      if (eligibiliteProjet(projet)) { continue }
      var projetRenvoye = {}
      var valeurs = []
      projetRenvoye.nom = projet.name
      projetRenvoye.identifiant = projet.identifier
      projetRenvoye.completion = {
        rempli: 0,
        non_rempli: 0
      }

      // On parcourt les colonnes une par une pour trouver les valeurs dans l'ordre
      for (var k = 0; k < CHAMPS_RETENUS.length; k++) {
        for (var j = 0; j < projet.custom_fields.length; j++) {
          if (projet.custom_fields[j].id == CHAMPS_RETENUS[k]) {
            valeurs.push(projet.custom_fields[j].value)
            if (projet.custom_fields[j].value == '') {
              projetRenvoye.completion.non_rempli++
            } else projetRenvoye.completion.rempli++
          }
        }
      }
      projetRenvoye.valeurs = valeurs
      listeProjets[projet.id] = projetRenvoye
    }
    if (presenceProjets(req, res, listeProjets) == 0) { return 0 }
    req.projets = listeProjets
    return next()
  })
};

// fonction permettant de trouver les noms de champ
function getNomChamps (req, res, next) {
  var nomsChamps = []
  redmine.custom_fields(function (err, data) {
    if (err) { console.log('[ERREUR REDMINE] '.red + err.red) }
    for (var j in CHAMPS_RETENUS) {
      for (var i in data.custom_fields) {
        if (data.custom_fields[i].id == CHAMPS_RETENUS[j]) {
          console.log('[CHAMP] ' + data.custom_fields[i].name)
          nomsChamps.push(data.custom_fields[i].name)
        }
      }
    }
    req.nomColonnes = nomsChamps
    return next()
  })
}

// fonction permettant de renvoyer le récapitulatif des projets
function renderRecapProjets (req, res) {
  res.render('tableauProjets', {
    title: 'Tableau des projets',
    listeProjets: req.projets,
    champs: req.typesChamps
  })
}

// fonction permettant de trouver les statistiques de remplissage des champs des projets
function trouveStats (req, res, next) {
  listeProjets = {}
  var remplis = {}
  var nonRemplis = {}

  redmine.projects({limit: MAX_PROJETS}, function (err, data) {
    if (err) { console.log('[ERREUR REDMINE] '.red + err.red) }
    var nomChamp
    for (var i in data.projects) {
      var projet = data.projects[i]
      if (eligibiliteProjet(projet)) { continue }
      listeProjets[i] = 1

      // On parcourt les colonnes une par une pour trouver les valeurs dans l'ordre
      for (var k = 0; k < CHAMPS_RETENUS.length; k++) {
        for (var j = 0; j < projet.custom_fields.length; j++) {
          nomChamp = projet.custom_fields[j].name

          if (projet.custom_fields[j].id == CHAMPS_RETENUS[k]) {
            // Si on n'a pas encore traité ce champ, on indique qu'il est à faire apparaître
            if ((!remplis.hasOwnProperty(nomChamp)) || (!nonRemplis.hasOwnProperty(nomChamp))) {
              remplis[nomChamp] = []
              nonRemplis[nomChamp] = []
            }
            // Si la value est remplie, on push dans 'remplis' le tableau qui correspond au champ
            if (projet.custom_fields[j].value != '') {
              remplis[nomChamp].push({nom: projet.name, id: projet.id})
            // Sinon on push dans 'nonRemplis'
            } else {
              nonRemplis[nomChamp].push({nom: projet.name, id: projet.id})
            }
          }
        }
      }
    }
    var stats = {
      'remplis': remplis,
      'nonRemplis': nonRemplis
    }
    if (presenceProjets(req, res, listeProjets) == 0) { return 0 } // besoin du req?
    req.stats = stats
    return next()
  })
};

// fonction permettant de trouver les types de champ
function getTypeChamps (req, res, next) {
  var typesChamps = []
  redmine.custom_fields(function (err, data) {
    if (err) { console.log('[ERREUR REDMINE] '.red + err.red) }
    for (var j in CHAMPS_RETENUS) {
      for (var i in data.custom_fields) {
        if (data.custom_fields[i].id == CHAMPS_RETENUS[j]) {
          console.log('[CHAMP] ' + data.custom_fields[i].name)
          if (data.custom_fields[i].field_format != 'list') {
            typesChamps.push({id: data.custom_fields[i].id, nom: data.custom_fields[i].name, format: data.custom_fields[i].field_format})
          } else { typesChamps.push({id: data.custom_fields[i].id, nom: data.custom_fields[i].name, format: data.custom_fields[i].field_format, multiple: data.custom_fields[i].multiple, possible_values: data.custom_fields[i].possible_values}) }
        }
      }
    }
    console.log('[TYPES CHAMPS] ' + JSON.stringify(typesChamps))
    req.typesChamps = typesChamps
    return next()
  })
}

// fonction qui traite la demande de création de projet
function traiterDemande (req, res, next) {
  var resultatDemande
  console.log('[FORMULAIRE] Envoyé: ' + JSON.stringify(req.body))
  var nomProjet = req.body.nomProjet
  var erreurRedmine = {}
  var cf = []
  cf.push({id: ID_CHAMP_ELIGIBLE, value: 1})

  for (var cle in req.body) {
    if (cle.indexOf('cf_') >= 0) {
      if (CHAMPS_RETENUS.includes(parseInt(cle.split(/_/)[1]))) {
        var valeur = req.body[cle]
        if (cle.split(/_/)[2] == 'd') {
          valeur = valeur.split('-').reverse().join('-')
        }
        if ((cle.split(/_/)[2] == 'b') && (valeur == 'on')) { valeur = '1' } // si c'est un booléen à "on", on remplace par "1"
        cf.push({id: cle.split(/_/)[1], value: valeur})
      }
    }
  }

  // faire la requête de création dans Redmine
  console.log('[ID GÉNÉRÉ] ' + nomProjet.toLowerCase().replace(/ /g, '-').sansAccent())
  redmine.create_project({project: {name: nomProjet, identifier: nomProjet.toLowerCase().replace(/ /g, '-').sansAccent(), custom_fields: cf}}, function (err, data) {
    if (err) {
      console.log('[ERREUR REDMINE] '.red + err.red)
      resultatDemande = 'ko'
      erreurRedmine = JSON.parse(err)
    } else {
      resultatDemande = 'ok'
    }
    req.erreurRedmine = erreurRedmine
    req.nomProjet = nomProjet
    req.resultatDemande = resultatDemande
    return next()
  })
};

// fonction qui trouve une page wiki
function trouvePage (req, res, next) {
  req.erreur = 0
  if (!req.params.projet) {
    req.erreur = 1
    req.contenuPage = '<div class="alert alert-warning">Veuillez sélectionner un projet.</div>'
    return next()
  }
  if (!req.params.page) {
    req.erreur = 1
    req.contenuPage = '<div class="alert alert-warning">Veuillez sélectionner une page.</div>'
    return next()
  }
  console.log('[WIKI] Page ' + String(req.params.page).blue + ' du projet ' + String(req.params.projet).blue)
  redmine.wiki_by_title(req.params.projet, req.params.page, {}, function (err, data) {
    if (err) {
      console.log('[ERREUR REDMINE] '.red + err.red)
      if (JSON.parse(err).hasOwnProperty('ErrorCode')) {
        req.erreur = 1
        req.contenuPage = '<div class="alert alert-danger">Page non trouvée.</div>'
        return next()
      }
    }
    console.log(textile(data.wiki_page.text))
    req.contenuPage = textile(data.wiki_page.text)
    return next()
  })
}

// fonction permettant de renvoyer les statistiques de remplissage des champs
function renderStats (req, res) {
  res.render('statistiquesProjets', {
    title: 'Statistiques des projets',
    champs: req.nomColonnes,
    stats: req.stats
  })
}

// fonction permettant de renvoyer un nouveau formulaire
function renderNouveauProjet (req, res) {
  res.render('nouveauProjet', {
    title: 'Nouveau Projet',
    message: 'Créer un nouveau projet',
    categories: req.categories,
    typesChamps: req.typesChamps
  })
}

function renderResultatDemande (req, res) {
  res.render('resultatDemande', {
    title: 'Création de projet',
    resultatDemande: req.resultatDemande,
    nomProjet: req.nomProjet,
    erreurRedmine: req.erreurRedmine
  })
}

app.post('/creerProjet', traiterDemande, renderResultatDemande)
app.get('/nouveauProjet', getTypeChamps, renderNouveauProjet)
app.get('/statistiques', trouveStats, getNomChamps, renderStats)
app.get('/recapitulatif', trouveProjets, getTypeChamps, renderRecapProjets)
app.get('/projets/:projet?', trouveProjets, getTypeChamps, function (req, res) {
  var projet = req.params.projet
  res.render(
    'projets',
    { title: "Résumé d'un projet",
      listeProjets: req.projets,
      typesChamps: req.typesChamps,
      projet: projet })
})
app.get('/wiki/:projet?/:page?', trouveProjets, trouvePage, function (req, res) {
  var projet = req.params.projet
  var page = req.params.page
  res.render(
    'wiki',
    { title: "Wiki d'un projet",
      listeProjets: req.projets,
      contenuPage: req.contenuPage,
      projet: projet,
      page: page,
      erreur: req.erreur
    })
})
app.get('/', function (req, res) {
  res.render(
    'index',
    { title: app.locals.appTitle, message: "Bienvenue sur l'application de gestion de projets" })
})
app.get('/testwiki', function (req, res) {
  redmine.wiki_by_title('depart-noumea', 'Logements', {}, function (err, data) {
    if (err) { console.log('[ERREUR REDMINE] '.red + err.red) }
    console.log(textile(data.wiki_page.text))
    res.send(textile(data.wiki_page.text))
  })
})

app.listen(PORT, function () {
  console.log('L\'application écoute sur le port ' + PORT + '.')
})
